/*-------------------------------------------------------------------------+
| Copyright 2018 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.extension.rumblepad;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.platform.raspberry.generator.extension.library.PiHALLibraryGeneratorExtensionBase;

/**
 * Base class for all executables of the rumblepad.
 * 
 * @author hoelzl
 */
abstract class RumblepadGeneratorExtensionBase<T extends EObject> extends
		PiHALLibraryGeneratorExtensionBase<T> {
	/** Constructor. */
	public RumblepadGeneratorExtensionBase(T modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonVariableDeclarationCode() {
		return "static rumblepad_configuration_t* rumblepad_config_" + getSingletonPostfix() +
				";\n";
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonAuxiliaryFunctions() {
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonInitializationCode() {
		String postfixedVar = "rumblepad_config_" + getSingletonPostfix();
		StringBuilder sb = new StringBuilder();
		sb.append("while(access(\"/dev/input/event0\", F_OK) == -1) {\n");
		sb.append("sleep(1);\n");
		sb.append("printf(\"Could not access gamepad device. Trying again in 1s.\\n\");\n");
		sb.append("}\n");
		sb.append("// initialize the rumblepad configuration\n");
		sb.append(postfixedVar + " = malloc(sizeof(rumblepad_configuration_t));\n");
		sb.append(postfixedVar + "->device_id = \"/dev/input/event0\";\n");
		sb.append(postfixedVar + "->waiting_sleep_in_micros = 50;\n");
		sb.append(postfixedVar + "->axis_callback = NULL;\n");
		sb.append(postfixedVar + "->button_callback = NULL;\n");
		sb.append("rumblepad_initialize(" + postfixedVar + ");\n\n");
		return sb.toString();
	}

	/** {@inheritDoc} */
	@Override
	public String getSingletonTerminationCode() {
		return "rumblepad_terminate();\n";
	}

	/** {@inheritDoc} */
	@Override
	public final String getSingletonIdentifier() {
		// use header file as singleton identifier
		return getHeaderFileName();
	}

	/** {@inheritDoc} */
	@Override
	public final String getHeaderFileName() {
		return "rumblepad.h";
	}
}
